@@ -687,7 +687,7 @@
 	 (t (sp-rational-quotient 1 ,x))))
 
 (defun sp-rat (x)
-  (cond ((or (polynomialp x)(rational-functionp x)) x)
+  (cond ((or (affine-polynomialp x)(rational-functionp x)) x)
 	(($bfloatp x) x)
 	(t (new-rat x))))
 
@@ -1109,7 +1109,7 @@ something is wrong" (length (sp-list-of-all-columns-occurring sp-mat)) number-of
 		  (cond ((not ($zerop  (aref (sp-constants-column sp-mat) i)))
 			 (format t "~%~%  ******************************~%")
 			 (format	  t "~%Row ~D has no pivot but the (sp-constants-column sp-mat) has entry ~D.
-		   ~%WARNING.  The equations were INCONSISTENT.
+		   ~%Warning: The equations were INCONSISTENT.
 		   ~%The special-solution is NOT VALID.
 		   ~%  ******************************" i (aref  (sp-constants-column sp-mat) i)))))
 	   (show (sp-type-of-entries sp-mat))
@@ -1133,8 +1133,7 @@ something is wrong" (length (sp-list-of-all-columns-occurring sp-mat)) number-of
 			   (vector-push
 			    (aref (sp-column-used-in-row sp-mat) nn) a-special-solution)
 			   (vector-push  a-new-entry a-special-solution)))))))
-    (cond ((and  (sp-solutions sp-mat)(ml-typep
-				       (sp-solutions sp-mat) 'sparse-matrix)))
+    (cond ((and  (sp-solutions sp-mat) (eq (type-of (sp-solutions sp-mat)) 'sparse-matrix)))
 	  (t (setf (sp-solutions sp-mat) (make-sparse-matrix ))))
     (sp-set-rows (sp-solutions sp-mat) solution-rows)
     (sp-set-type-of-entries (sp-solutions sp-mat)
@@ -1432,7 +1431,7 @@ something is wrong" (length (sp-list-of-all-columns-occurring sp-mat)) number-of
 ;	      (t (cond (($numberp (sp-pivot-entry sp-mat))
 ;			(setf (sp-pivot-entry sp-mat)
 ;			      (cond ((atom (sp-pivot-entry sp-mat))(sp-pivot-entry sp-mat))
-;				    ((or (polynomialp (sp-pivot-entry sp-mat))
+;				    ((or (affine-polynomialp (sp-pivot-entry sp-mat))
 ;					 (rational-functionp (sp-pivot-entry sp-mat)))
 ;				     (sp-pivot-entry sp-mat))
 ;				    (t
@@ -2164,7 +2163,7 @@ something is wrong" (length (sp-list-of-all-columns-occurring sp-mat)) number-of
 (defun float-zerop (n)
   (cond ((numberp n) (or (zerop n)(and (floatp n) (< (abs n) .0001))))
 	((atom n) nil)
-	((polynomialp n) nil)
+	((affine-polynomialp n) nil)
 	((rational-functionp n)(float-zerop (car n)))
 	(t
 	 (let (tem type-of-n)
